//
// bignum_common.hpp
//
// Bill Seymour, 2024-02-02
//
// Copyright Bill Seymour 2024.
// Distributed under the Boost Software License, Version 1.0.
// (See accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)
//
// This header declares an unscoped enumeration of rounding modes.
// At present, it's used only by the decimal and rational classes.
// It's borrowed from WG21's P1889R1 sect. 3.3; but it's unscoped
// putting the enumerators in the bignum namespace.
//
// Also defined are templates for human-readable names of built-in
// types.  (std::type_info::name() isn't always very informative.)
// At present, these are used only by the integer class; but they're
// referenced in integer.hpp which is #included in both decimal.hpp
// and rational.hpp.
//
// Also declared is a function defined in bignum_common.cpp that
// throws a std::overflow_error.
//

#ifndef BIGNUM_COMMON_HPP_INCLUDED
#define BIGNUM_COMMON_HPP_INCLUDED

namespace bignum {

enum rounding
{
    all_to_neg_inf, all_to_pos_inf,
    all_to_zero, all_away_zero,
    all_to_even, all_to_odd,
    all_fastest, all_smallest,
    all_unspecified,
    tie_to_neg_inf, tie_to_pos_inf,
    tie_to_zero, tie_away_zero,
    tie_to_even, tie_to_odd,
    tie_fastest, tie_smallest,
    tie_unspecified
};

namespace types {

template<typename T> struct name
{
    static constexpr char const * const value = "unknown type";
};
template<> struct name<bool>
{
    static constexpr char const * const value = "bool";
};
template<> struct name<char>
{
    static constexpr char const * const value = "char";
};
template<> struct name<signed char>
{
    static constexpr char const * const value = "signed char";
};
template<> struct name<unsigned char>
{
    static constexpr char const * const value = "unsigned char";
};
template<> struct name<short>
{
    static constexpr char const * const value = "short";
};
template<> struct name<unsigned short>
{
    static constexpr char const * const value = "unsigned short";
};
template<> struct name<int>
{
    static constexpr char const * const value = "int";
};
template<> struct name<unsigned int>
{
    static constexpr char const * const value = "unsigned int";
};
template<> struct name<long>
{
    static constexpr char const * const value = "long";
};
template<> struct name<unsigned long>
{
    static constexpr char const * const value = "unsigned long";
};
template<> struct name<long long>
{
    static constexpr char const * const value = "long long";
};
template<> struct name<unsigned long long>
{
    static constexpr char const * const value = "unsigned long long";
};
template<> struct name<float>
{
    static constexpr char const * const value = "float";
};
template<> struct name<double>
{
    static constexpr char const * const value = "double";
};
template<> struct name<long double>
{
    static constexpr char const * const value = "long double";
};

[[noreturn]] void throw_overflow(const char*, const char*);

} } // namespace bignum::types

#endif // BIGNUM_COMMON_HPP_INCLUDED
